#include <sstream>
#include <iostream>
#include <stdexcept>
#include <math.h>

#include "NemoMath/Vector.h"

class DataTransferObject {

public:

    virtual std::string print() const {
        std::string str;
        str = "<DataTransferObject>";
        return str;
    }

    friend std::ostream
    & operator<<(std::ostream& os, const DataTransferObject& dto);

protected:

    DataTransferObject() {
    }

    virtual ~DataTransferObject() {
    }
};

class ContainingDoubles {

public:

    virtual unsigned int getDimension() const {
        return this->_dimension;
    }

    virtual double asDouble() const {
        if (this->_dimension == 1) {
            return this->asDouble(0);
        } else {
            throw std::invalid_argument(
                    "Data transfer object multi-dimensional, but no dimension given.");
        }
    }

    virtual double asDouble(unsigned int index) const {
        if (index >= this->_dimension) {
            throw std::invalid_argument("Index exceeds dimension.");
        }
        return this->_values[index];
    }

    virtual nemo::RealVector asDoubleVector() const {
        return this->_values;
    }

protected:

    ContainingDoubles() :
        _dimension(), _values() {
    }

    ContainingDoubles(unsigned int dimension) :
        _dimension(dimension), _values(dimension) {
    }

    ContainingDoubles(nemo::RealVector values) :
        _dimension(values.dimension()), _values(values) {
    }

    virtual ~ContainingDoubles() {
    }

    unsigned int _dimension;
    nemo::RealVector _values;
};

class DebugDTO: public DataTransferObject, public ContainingDoubles {

public:
    DebugDTO() :
        DataTransferObject(), ContainingDoubles() {
    }

    DebugDTO(nemo::RealVector values) :
        DataTransferObject(), ContainingDoubles(values) {
        this->_values = values;
    }

    virtual std::string print() const {
        std::ostringstream outstream(std::ostringstream::out);
        outstream.precision(3); // Precision when printing double values
        outstream << "DebugDTO";
        return outstream.str();
    }

};

std::ostream& operator<<(std::ostream& os, const DataTransferObject& dto) {
    os.precision(3); // Precision when printing double values
    os << dto.print();
    return os;
}


int main() {
    nemo::RealVector fu = nemo::RealVector(3);
    DebugDTO foo(fu);
}

