#include "Beat.h"

using namespace std;
using namespace boost;

namespace cca {

GlobalBeat::GlobalBeat(unsigned int milliseconds) :
    _samplerate(milliseconds), _nextWakeup(), _m(), _cond(), _lk(_m),
            _done(false), _aborted(false), _receiver() {
}

void GlobalBeat::run() {
    _nextWakeup = get_system_time() + posix_time::milliseconds(_samplerate);
    if (DF_DEBUG) {
        std::cout << "Start ticking ..." << std::endl;
    }
    while (!_done && !_aborted) {
        if (!_cond.timed_wait(_lk, _nextWakeup)) {
            _nextWakeup = get_system_time() + posix_time::milliseconds(
                    _samplerate);
            if (DF_DEBUG && DF_VERBOSITY > 0) {
                std::cout << "------------- tick -------------" << std::endl;
            }

            // Send tick to registered receivers
            // TODO: Iterator
            for (unsigned int index = 0; index < _receiver.size(); ++index) {
                _receiver[index]->tick();
            }
        }
    }
}

void GlobalBeat::stop() {
    if (DF_DEBUG) {
        std::cout << "Global beat stopped." << std::endl;
    }
    _aborted = true;
}

GlobalBeat::~GlobalBeat() {

}

void GlobalBeat::registerReceiver(CCANodePtr receiver) {
    if (receiver->isTimed()) {
        _receiver.push_back(receiver);
    }
}

string GlobalBeat::print() const {
    ostringstream outstream(ostringstream::out);
    outstream.precision(3); // Precision when printing double values
    outstream << "<GlobalBeat (frequency=" << this->_samplerate << "ms)>"
            << endl;
    return outstream.str();
}

GlobalBeatPtr GlobalBeat::create(unsigned int milliseconds) {
    GlobalBeatPtr beat(new GlobalBeat(milliseconds));
    return beat;
}

ostream& operator<<(ostream& os, const GlobalBeat& beat) {
    os.precision(3); // Precision when printing double values
    os << beat.print();
    return os;
}

}
