#pragma once

#include <string>
#include <sstream>
#include <iostream>

#include <boost/bind.hpp>
#include <boost/signal.hpp>
#include <boost/thread.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>

#include "cca/nodes/CCANode.h"
#include "cca/Config.h"

namespace cca {

class GlobalBeat;
typedef boost::shared_ptr<GlobalBeat> GlobalBeatPtr;

/**
 * Global heart beat of the Data-Flow graph. With the specified samplerate, all
 * registered node get a global beat signal.
 */
class GlobalBeat {
public:
    GlobalBeat(unsigned int milliseconds);
    ~GlobalBeat();

    void run();
    void stop();

    std::string print() const;

    void registerReceiver(CCANodePtr receiver);
    CCANodePtr getReceivers();

    /**
     * Factory
     */
    static GlobalBeatPtr
    create(unsigned int milliseconds = 10);

    /**
     * Stream operator
     * @param os
     * @param node
     * @return
     */
    friend std::ostream
    & operator<<(std::ostream& os, const GlobalBeat& beat);
private:
    /**
     * Samplerate in milliseconds. Each _samplerate milliseconds, alls receivers
     * get the global beat signal.
     */
    unsigned int _samplerate;

    boost::system_time _nextWakeup;
    boost::mutex _m;
    boost::condition_variable _cond;
    boost::unique_lock<boost::mutex> _lk;
    bool _done, _aborted;

    /**
     * Vector of registered CCANodes to receive the global beat signal.
     */
    std::vector<CCANodePtr> _receiver;
};

}
