#pragma once

#include <string>
#include <iostream>
#include <boost/date_time/posix_time/posix_time.hpp>

// FastFlow
#include "ff/node.hpp"

#include "cca/dto/Task.h"
#include "cca/Config.h"

namespace cca {

class CCANode;
typedef boost::shared_ptr<CCANode> CCANodePtr;

/**
 * Base Data-Flow Node
 * Now configurable if timed or asynchronous node.
 * @todo we probably want to have two node types (timed and asynchronous) as DIC
 * later, where the actual functionality is injected.
 */
class CCANode: public ff::ff_node {

public:

    /**
     * Constructor
     * @param nodename Name of this Node (sko. ID)
     * @param timed Is this a timed node (alternative: asynchronous node)
     * @param cycles If timed node, number of cycles to process
     */
    CCANode(std::string nodename, bool timed = false,
            unsigned int samplerate = 1);
    ~CCANode();

    /**
     * svc method of the asynchronous node just needs to call the process method.
     * This is only for that
     * @todo Maybe handle the DebugDTO here so it gets handled by all nodes?
     */
    virtual void* svc(void *task);

    /**
     * Returns name od the node
     */
    virtual std::string getName();

    /**
     * Returns if node is ready for processing.
     */
    virtual bool isReady() const;

    /**
     * Returns if this node is a timed node.
     * Not sure if we really need this.
     */
    virtual bool isTimed() const;

    /**
     * For global synchonization, this node receives a global clock beat.
     */
    void tick();

    /**
     * Function called with configured sample rate.
     */
    virtual void onBeat();

    /**
     * Printing information about this node
     */
    virtual std::string print() const = 0;

protected:

    /**
     * This is the mathod where the actual processing of the node takes place.
     * @param task The task to be processed by this node
     * @return The processing result
     */
    virtual void* process(void *task);

    std::string _nodename;
    bool _timed;
    unsigned int _sampleRate;
    unsigned int _taskcounter;
    unsigned int _svccounter;
    unsigned int _sentCounter;
    unsigned int _ticksCounter;
    unsigned int _ticksCounterAll;
    bool _ready;

    /**
     * Buffering the last processed result. In case of timed node.
     * @todo Use FastFlow out buffer
     */
    void *_buffer;

    /**
     * Buffering the last processed result. In case of timed node.
     * @todo Use FastFlow out buffer
     */
    void *_bufferOld;

private:
    /**
     * Copy constructor
     * Copy constructor private for now to disallow copying.
     */
    CCANode(CCANode&m);

    /**
     * Copy operator
     * Copy operator private for now to disallow copying.
     */
    void operator=(const CCANode&);
};

/**
 * Stream operator
 * @param os
 * @param node
 * @return
 */
std::ostream & operator<<(std::ostream& os, const CCANode& node);

}
