#include "Collector.h"

using namespace ff;

namespace cca {

Collector::Collector(std::string nodename, unsigned int numInputNodes, bool timed,
        unsigned int samplerate) :
    MCSPNode(nodename, numInputNodes, timed, samplerate), _status(0, 0, 0) {
}

Collector::~Collector() {
}

void* Collector::process(void *task) {
    std::cout << "Collector >" << _nodename << "<::process" << std::endl;

    if (task == EOS) {
        std::cout << " ++++ Collector >" << _nodename << "< received EOS."
                << std::endl;
        return EOS;
    }

    void *returntask;
    returntask = this->collectTasks();

    ThreeValues *status = static_cast<ThreeValues*> (returntask);
    std::cout << " ++++ Collector >" << _nodename << "< send status "
            << *status;

    return returntask;
}

void* Collector::collectTasks() {
    std::cout << "Collector>" << _nodename << "<::collectTasks" << std::endl;
    CCANode *resource;
    usleep(10000);

    void *crt;
    void **crtpointer = &crt;

    if (!_prepared) {
        this->prepare();
    }

    for (unsigned int i = 0; i < 3; ++i) {
        // The input node, we want to adress
        resource = static_cast<CCANode*> (_inputNodes[i]);

        // Trigger processing of the node so it fetches the current status
        //resource->svc(GO_ON);
        if (resource->out->empty()) {

            continue;
        }

        // Get task from input node
        ++_taskcounter;
        resource->get(crtpointer);

        // Check if EOS was sent, in that case abort
        if (*crtpointer == EOS) {
            std::cout << " ++------++ Collector >" << _nodename
                    << "< received EOS." << std::endl;
            return EOS;
        }
        if (*crtpointer == NULL) {
            std::cout << " ++----------++ Collector >" << _nodename
                    << "< received NULL." << std::endl;
            return GO_ON;
        }
        if (*crtpointer == GO_ON) {
            std::cout << " ++----------++ Collector >" << _nodename
                    << "< received NULL." << std::endl;
            return GO_ON;
        }

        // Try to cast
        Task *t;
        t = static_cast<Task*> (*crtpointer);
        if (t->empty()) {
            std::cerr << "Collector >" << _nodename
                    << "< collected empty task from node >"
                    << resource->getName() << "<." << std::endl;
        } else {
            SingleValue *sstatus = static_cast<SingleValue*> (*crtpointer);
            std::cout << "Collector >" << _nodename
                    << "< collected task with content >" << sstatus->getValue()
                    << "< from node >" << resource->getName() << "<"
                    << std::endl;
            _status.setValue(i, sstatus->getValue());
        }
    }

    // Return current statusstd::cerr << "Collector >" << _nodename
    std::cerr << "Collector >" << _nodename
            << "< crafted a three values status: " << _status;
    void *rtn = static_cast<void*> (&_status);
    return rtn;
}

void Collector::registerInputNode(CCANode* inputNode) {
    if (_nInputNodes < _maxInputNodes) {
        _inputNodes.push_back(inputNode);
        ++_nInputNodes;
        if (DF_DEBUG && DF_VERBOSITY > 2) {
            std::cout << "Collector >" << _nodename << "< added input node >"
                    << inputNode->getName() << "<" << std::endl;
        }

    } else {
        std::cerr << " Collector>" << _nodename
                << "< can't add any more input nodes." << std::endl;
    }
}

std::ostream& operator<<(std::ostream& os, const Collector& node) {
    os.precision(3); // Precision when printing double values
    os << std::endl << "Collector >" << node._nodename << "<" << std::endl;
    os << " called " << node._svccounter << " times" << std::endl;
    os << " received " << node._taskcounter << " tasks" << std::endl;
    os << " received from " << node._nInputNodes << " registered inputs"
            << std::endl;
    return os;
}

}
