#pragma once

#include <string>
#include <iostream>

// Include FatFlow gatherer to have all necessary tools included
#include <ff/node.hpp>
#include <ff/farm.hpp>
#include <ff/gt.hpp>

#include "cca/Config.h"
#include "CCANode.h"

namespace cca {

/**
 * Single-Consumer Multi-Producer Node
 * Receiving tasks by one single node, sending tasks to several nodes. Simple,
 * just forwarding the received task without any processing.
 * @todo Use FastFlow load balancer
 */
class MCSPNode: public CCANode {
public:
    MCSPNode(std::string nodename, unsigned int max_input_nodess = 10,
            bool timed = false, unsigned int samplerate = 1);
    ~MCSPNode();

    /**
     * Add one more receiver
     * Add one more receiver to the list of receivers.
     * @param Node to be connected
     */
    virtual void registerInputNode(CCANode *inputNode);

    /**
     * Add one more receiver
     * Add one more receiver to the list of receivers.
     * @param position register this node at a certain position
     * @param position node to register as input
     */
    virtual void registerInputNode(unsigned int position,
            CCANode *inputNode);

    /**
     * Add a set of receivers
     * Add a set of receivers to the list of receivers.
     * @param Vector of nodes to be connected
     */
    virtual void registerInputNodes(std::vector<CCANode *> &inputNodes);

    /**
     * The actual working
     * This could mean just passing the task to all registered _receivers.
     */
    virtual void* process(void *task);

    /**
     * Stream operator
     * @param os
     * @param node
     * @return
     */
    friend std::ostream
    & operator<<(std::ostream& os, const MCSPNode& node);

protected:
    /**
     * Prepare input and output buffers
     */
    virtual void prepare();

    /**
     * Collect tasks from all registered inputs
     */
    virtual void* collectTasks();

    unsigned int _nInputNodes, _maxInputNodes;
    std::vector<CCANode *> _inputNodes;
    bool _prepared;
    ff::FFBUFFER* _outputBuffer;

private:
    /**
     * Copy constructor
     * Copy constructor private for now to disallow copying.
     */
    MCSPNode(MCSPNode&m);

    /**
     * Copy operator
     * Copy operator private for now to disallow copying.
     */
    void operator=(const MCSPNode&);
};

}
