#include "SCMP.h"

using namespace ff;

namespace cca {

SCMPNode::SCMPNode(std::string nodename, unsigned int max_receivers) :
    CCANode(nodename), _nReceivers(0), _maxReceivers(max_receivers),
            _receivers(), _outs(), _prepared(false) {
}
SCMPNode::SCMPNode(std::string nodename, bool timed, unsigned int samplerate,
        unsigned int max_receivers) :
    CCANode(nodename, timed, samplerate), _nReceivers(0),
            _maxReceivers(max_receivers), _receivers(), _outs(),
            _prepared(false) {
}

SCMPNode::~SCMPNode() {
}

int SCMPNode::svc_init() {
    if (_prepared) {
        return 0;
    }
    this->prepare();
    return 1;
}

void SCMPNode::prepare() {
    std::cout << "SCMPNode >" << _nodename << "<::prepare" << std::endl;
    if (!_prepared) {

        // Output buffer for every receiver
        for (unsigned int i = 0; i < _nReceivers; ++i) {
            if (this->_receivers[i]->create_input_buffer(10) < 0) {
                std::cout << "SCMPNode >" << _nodename
                        << ": Error while creating input buffer for receiver"
                        << i << std::endl;
            }
        }

        for (unsigned int i = 0; i < _nReceivers; ++i) {
            this->_outs.push_back(this->_receivers[i]->get_in_buffer());
        }

        for (unsigned int i = 0; i < _nReceivers; ++i) {
            this->_receivers[i]->set_id((int) i + 2);
            this->_receivers[i]->run(false);
        }

        this->_prepared = true;
    }
}

void* SCMPNode::process(void *task) {
    if (DF_DEBUG && DF_VERBOSITY > 2) {
        std::cout << "SCMPNode >" << _nodename << "<::process" << std::endl;
    }
    ++_taskcounter;

    if (!_prepared) {
        this->prepare();
    }

    // Broadcast of this task to all registered receivers
    for (unsigned int i = 0; i < _nReceivers; ++i) {
        this->_outs[i]->push(task);
        this->_receivers[i]->svc(task);
    }

    // FastFlow macro to go on with data-flow without doing anything
    return task;
}

void SCMPNode::addReceiver(CCANodePtr receiver) {
    if (DF_DEBUG && DF_VERBOSITY > 2) {
        std::cout << "SCMPNode >" << _nodename << "<::addReceiver >"
                << receiver->getName() << "<" << std::endl;
    }
    _receivers.push_back(receiver);
    ++_nReceivers;
}

void SCMPNode::addReceivers(std::vector<CCANodePtr> receivers) {
    for (unsigned int i = 0; i < receivers.size(); ++i) {
        this->addReceiver(receivers[i]);
    }
}

std::ostream& operator<<(std::ostream& os, const SCMPNode& node) {
    os.precision(3); // Precision when printing double values
    os << std::endl << "SCMPNode >" << node._nodename << "<" << std::endl;
    os << " called " << node._svccounter << " times" << std::endl;
    os << " received " << node._taskcounter << " tasks" << std::endl;
    os << " sent to " << node._nReceivers << " registered receivers"
            << std::endl;
    return os;
}

}
