#pragma once

#include <string>
#include <iostream>

// FastFlow
#include <ff/node.hpp>

#include "cca/Config.h"
#include "CCANode.h"

namespace cca {

/**
 * Single-Consumer Multi-Producer Node
 * Receiving tasks by one single node, sending tasks to several nodes. Simple,
 * just forwarding the received task without any processing.
 * @todo Use FastFlow load balancer
 */
class SCMPNode: public CCANode {
public:
    SCMPNode(std::string nodename, unsigned int max_receivers);
    SCMPNode(std::string nodename, bool timed = false,
            unsigned int samplerate = 1, unsigned int max_receivers = 10);
    ~SCMPNode();

    /**
     * Add one more receiver
     * Add one more receiver to the list of receivers.
     * @param Node to be connected
     */
    virtual void addReceiver(CCANodePtr receiver);

    /**
     * Add a set of receivers
     * Add a set of receivers to the list of receivers.
     * @param Vector of nodes to be connected
     */
    virtual void addReceivers(std::vector<CCANodePtr> receivers);

    virtual int svc_init();

    /**
     * The actual working
     * This could mean just passing the task to all registered _receivers.
     */
    virtual void* process(void *task);

    /**
     * Stream operator
     * @param os
     * @param node
     * @return
     */
    friend std::ostream
    & operator<<(std::ostream& os, const SCMPNode& node);

protected:

    virtual void prepare();

    unsigned int _nReceivers, _maxReceivers;
    std::vector<CCANodePtr> _receivers;
    std::vector<ff::FFBUFFER*> _outs;
    bool _prepared;
};

}
