#include "Splitter.h"

using namespace ff;

namespace cca {

Splitter::Splitter(std::string nodename, unsigned int num_receivers,
        bool timed, unsigned int samplerate) :
    SCMPNode(nodename, timed, samplerate, num_receivers) {
}

Splitter::~Splitter() {
}

void Splitter::prepare() {
    // Input buffer
}

void* Splitter::process(void *task) {
    if (DF_DEBUG && DF_VERBOSITY > 2) {
        std::cout << "Splitter >" << _nodename << "<::process" << std::endl;
    }
    ++_taskcounter;

    if (task == NULL) {
        std::cout << "Splitter >" << _nodename << "< received NULL."
                << std::endl;
        return NULL;
    }
    if (task == EOS) {
        std::cout << "Splitter >" << _nodename << "< received EOS."
                << std::endl;
        return EOS;
    }

    // Split ComplexTask into SingleTasks
    ComplexTask *ct = static_cast<ComplexTask*> (task);
    if (ct->getDimension() != this->_maxReceivers) {
        std::cout << "Splitter >" << _nodename
                << "<: Task dimension doesn`t agree." << std::endl;
        return EOS;
    }

    std::cout << "Splitter >" << _nodename << "< now splitting tasks."
            << std::endl;
    SingleTask *st1, *st2, *st3;
    st1 = new SingleTask(ct->getValue(0));
    st2 = new SingleTask(ct->getValue(1));
    st3 = new SingleTask(ct->getValue(2));

    // Broadcast of this task to all registered receivers
    _receivers[0]->svc(st1);
    _receivers[1]->svc(st2);
    _receivers[2]->svc(st3);

    // FastFlow macro to go on with data-flow without doing anything
    return GO_ON;
}

std::ostream& operator<<(std::ostream& os, const Splitter& node) {
    os.precision(3); // Precision when printing double values
    os << std::endl << "Splitter >" << node._nodename << "<" << std::endl;
    os << " called " << node._svccounter << " times" << std::endl;
    os << " received " << node._taskcounter << " tasks" << std::endl;
    os << " sent to " << node._nReceivers << " registered receivers"
            << std::endl;
    return os;
}

}
