#pragma once

#include <string>
#include <iostream>

// FastFlow
#include <ff/lb.hpp>

#include "cca/Config.h"
#include "SCMP.h"

namespace cca {

/**
 * Single-Consumer Multi-Producer Node
 * Receiving tasks by one single node, sending tasks to several nodes. Simple,
 * just forwarding the received task without any processing.
 * @todo Use FastFlow load balancer
 */
class Splitter: public SCMPNode {
public:
    Splitter(std::string nodename, unsigned int num_receivers,
            bool timed = false, unsigned int samplerate = 1);
    ~Splitter();

    /**
     * The actual working
     * This could mean just passing the task to all registered _receivers.
     */
    virtual void* process(void *task);

    /**
     * Stream operator
     * @param os
     * @param node
     * @return
     */
    friend std::ostream
    & operator<<(std::ostream& os, const Splitter& node);

protected:
    /**
     * Prepare input and output buffers
     */
    virtual void prepare();
};

}
