#pragma once

#include <string>
#include <iostream>

#include "cca/Config.h"
#include "cca/nodes/CCANode.h"
#include "cca/dto/debug/DebugDTO.h"

namespace cca {

class RandomNumberGenerator;
typedef boost::shared_ptr<RandomNumberGenerator> RandomNumberGeneratorPtr;

/**
 * RandomNumberGenerator
 * RandomNumberGenerator for debug purposes just outputs a data-flow stream of
 * random numbers.
 */
class RandomNumberGenerator: public CCANode {
public:
    RandomNumberGenerator(std::string nodename, bool timed = false,
            unsigned int samplerate = 1);
    ~RandomNumberGenerator();

    /**
     * Set range for the random numbers.
     * @param lower Lower bound of random number range, lowest possible output
     * @param upper Upper bound of random number range, highest possible output
     */
    void setRange(double lower, double upper);

    /**
     * Limit number of random numbers sent. After that, there will be an EOS
     * sent to end the program.
     */
    void limitNumber(unsigned int number);

    /**
     * Printing this node
     */
    std::string print() const;

    /**
     * Shared Pointer Factory
     */
    static RandomNumberGeneratorPtr create(std::string name = "RNG",
            bool timed = false, unsigned int samplerate = 1);

    /**
     * Assign to the beat to count numbers of sent random numbers.
     */
    void onBeat();

protected:

    /**
     * The actual working
     * This could mean just passing the task to all registered _receivers.
     */
    virtual void* process(void *task);

    /**
     * Ranges for random numbers
     */
    double _lower, _upper;

    /**
     * Optional: Number of random data sets to send
     */
    int _number;
};

}
